package com.zerog.ia.customcode.actions;

import com.zerog.ia.api.pub.*;
import java.net.*;
import java.io.*;

/**
 * EnhancedExecuteCommand.java
 *
 * This custom code action execute a command, and then captures it's output and return code.
 * It also sets the captured output and return code from the command, and sets InstallAnywhere Variables
 */
public class EnhancedExecuteCommand extends CustomCodeAction
{
	/**
	 *  This is the method that is called at install-time.  The InstallerProxy
	 *  instance provides methods to access information in the installer,
	 *  set status, and control flow.
	 */
	public void install( InstallerProxy ip ) throws InstallException
	{

		try
		{
		   // Get the value of the "$COMMAND_TO_EXECUTE$" InstallAnywhere variable
		   String executeString = ip.substitute("$COMMAND_TO_EXECUTE$");

		   // Set up the command to run.
		   //System.out.println("EnhancedExecuteCommand: About to execute: "+executeString );
	       final Process p = Runtime.getRuntime().exec(executeString);

	       // Intialize StringWriters that capture the output from the command
	       final StringWriter errWriter = new StringWriter();
	       final StringWriter outWriter = new StringWriter();

	       // Launch a thread to capture the Standard Error Output of the Command
		   Thread thread = new Thread()
		   {
		      public void run()
		      {
		         InputStream err = p.getErrorStream();
		         int c;
		         try
		         {
		            while ((c = err.read()) >= 0)
		            {
		               errWriter.write(c);
		            }
		         }
		         catch (IOException ioe)
		         {
		         }
		      }
		   };

	       // Launch a thread to capture the Standard Output of the Command
		   Thread thread2 = new Thread()
		   {
		      public void run()
		      {
		         InputStream err = p.getInputStream();
		         int c;
		         try
		         {
		            while ((c = err.read()) >= 0)
		            {
		               outWriter.write(c);
		            }
		         }
		         catch (IOException ioe)
		         {
		         }
		      }
		   };

		   thread.start();
		   thread2.start();
	       p.waitFor();

		   // Take the information captured from the string writers, and write them out.
		   String stdErr   = errWriter.toString();
		   String stdOut   = outWriter.toString();

		   // Get the return code
		   String exitCode = new Integer(p.exitValue()).toString();

		   // Print out captured info (for debugging purposes)
		   // System.out.println("EnhancedExecuteCommand: StdErr Output: "+stdErr );
		   // System.out.println("EnhancedExecuteCommand: StdOut Output: "+stdOut );
		   // System.out.println("EnhancedExecuteCommand: Exit Code    : "+exitCode );

		   // Set the InstallAnywhere variables
           ip.setVariable("CC_STD_ERR",   stdErr);
           ip.setVariable("CC_STD_OUT",   stdOut);
           ip.setVariable("CC_EXIT_CODE", exitCode);
		}
		catch(Exception e)
		{
			throw new NonfatalInstallException("Execute Command Custom Code Failed while reading Error Stream");
		}
	}


	/**
	 *  This is the method that is called at uninstall-time.  The DataInput
	 *  instance provides access to any information written at install-time
	 *  with the instance of DataOutput provided by UninstallerProxy.getLogOutput().
	 */
	public void uninstall( UninstallerProxy up ) throws InstallException
	{
	}

	/**
	 *  This method will be called to display a status message during the
	 *  installation.
	 */
	public String getInstallStatusMessage()
	{
		return "Execute Command";	// screen will say "Installing... My Action"
	}

	/**
	 *  This method will be called to display a status message during the
	 *  uninstall.
	 */
	public String getUninstallStatusMessage()
	{
		return "Execute Command";	// screen will say "Uninstalling... My Action"
	}
}